pub struct BackendCapabilities {
pub transactional_enqueue: bool,
pub durable_jobs: bool,
pub notifications: bool,
pub streams: bool,
pub consumer_groups: bool,
pub distributed_workers: bool,
pub ordering: OrderingCapability,
pub backpressure: BackpressureCapability,
}Expand description
Storage backend feature and guarantee declaration.
Capabilities describe what a backend can honestly provide. They are not a marketing matrix: application code can inspect this value when it needs a specific storage guarantee.
Fields
transactional_enqueue: booldurable_jobs: boolnotifications: boolstreams: boolconsumer_groups: booldistributed_workers: boolordering: OrderingCapabilitybackpressure: BackpressureCapabilityImplementations
Sourceimpl BackendCapabilities
impl BackendCapabilities
pub const fn memory() -> Self
pub const fn sqlite() -> Self
pub const fn postgres() -> Self
pub const fn redis() -> Self
pub fn supports_portable_job_api(&self) -> bool
Sourcepub const fn semantics(&self) -> Option<BackendSemanticCapabilities>
pub const fn semantics(&self) -> Option<BackendSemanticCapabilities>
Returns the detailed profile for an exact built-in capability declaration.
Unknown custom combinations return None instead of being guessed as a built-in backend.