pub struct JobExecution {
pub job_id: Uuid,
pub attempt_id: Uuid,
pub attempt_no: i32,
pub worker_id: String,
pub lease_expires_at: DateTime<Utc>,
pub started_at: DateTime<Utc>,
}Expand description
Runtime execution claim tying a job, durable attempt, worker, and lease together.
JobExecution is the in-flight view of work. The durable record of the handler run is
JobAttempt; the durable record of the work item is Job.
Fields
job_id: Uuidattempt_id: Uuidattempt_no: i32worker_id: Stringlease_expires_at: DateTime<Utc>started_at: DateTime<Utc>Trait Implementations
Sourceimpl Clone for JobExecution
impl Clone for JobExecution
Sourcefn clone(&self) -> JobExecution
fn clone(&self) -> JobExecution
Returns a duplicate of the value. Read more
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl Debug for JobExecution
impl Debug for JobExecution
Sourceimpl<'de> Deserialize<'de> for JobExecution
impl<'de> Deserialize<'de> for JobExecution
Sourcefn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
Deserialize this value from the given Serde deserializer. Read more
impl Eq for JobExecution
Sourceimpl PartialEq for JobExecution
impl PartialEq for JobExecution
Sourceimpl Serialize for JobExecution
impl Serialize for JobExecution
impl StructuralPartialEq for JobExecution