Module semantics

Module semantics 

Structs

SemanticContract
Machine-readable answer to “what does Azums guarantee for this behavior?”.

Enums

SemanticBehavior
A public Azums behavior whose contract is stable and explicitly classified.
SemanticClassification
Stability class of a documented Azums behavior.

Functions

semantic_contract
Returns the canonical product contract for every public semantic behavior.