pub const fn semantic_contract(behavior: SemanticBehavior) -> SemanticContractExpand description
Returns the canonical product contract for every public semantic behavior.
pub const fn semantic_contract(behavior: SemanticBehavior) -> SemanticContractReturns the canonical product contract for every public semantic behavior.